; setup our Screen Width and Height here for easier tracking
#ScreenWidth  = 640
#ScreenHeight = 480

; Initialize the sprite and keyboard, sound, and a 640x480, 16-bit screen
If InitSprite() = 0 Or InitKeyboard() = 0 Or InitSound() = 0 Or OpenScreen(#ScreenWidth,#ScreenHeight,16,"App Title") = 0
   MessageRequester("Error!", "Unable to Initialize Environment", #PB_MessageRequester_Ok)
   End
EndIf

ClearColor = RGB(0,0,0)

; setup our structure for the sounds
Structure SoundController
  ID.l 
  Length.l 
  StartTime.l
EndStructure

; make sure we have a list for the sounds
Global NewList Sounds.SoundController()

 ;****************************************************
 ;  Procedure: MyPlaySound()
 ;     Author: Derlidio Siqueira & John Logsdon
 ;   Last Upd: 2/07/2005
 ;    Purpose: Adds a sound to the Sounds list and
 ;             randomizes it's volume, pan, and frequency values
 ;       Args: n/a
 ;    Returns: n/a
 ;   Comments: None
 ;****************************************************
 Procedure MyPlaySound()
  
    ; first grab the sound out of memory
    ID = CatchSound(#PB_Any,?ExplosionSound)

    ; then set the approximate time for the sound
    Length.l = 5000
   
    ; if a valid ID is found
    If ID
      ; add the sound to the list
      AddElement(Sounds())
      
      ; then populate the list
      Sounds()\ID = ID
      Sounds()\Length = Length.l
      Sounds()\StartTime = ElapsedMilliseconds()

      ; set up some randomness for effect
      Volume = Random(50) + 50
      SoundVolume(ID,Volume)
      Pan = Random(200) - 100
      SoundPan(ID,Pan)
      Frequency = Random(30000) + 30000
      SoundFrequency(ID,Frequency)

      ; play the sound
      PlaySound(ID)
      
    EndIf
    
  EndProcedure
  
  ;****************************************************
  ;  Procedure: MyReleaseSound()
  ;     Author: Derlidio Siqueira
  ;   Last Upd: 2/07/2005
  ;    Purpose: Removes a sound from the Sounds list
  ;       Args: n/a
  ;    Returns: n/a
  ;   Comments: This will kill a sound regardless if
  ;             it is still playing or not. It kills
  ;             the sound based on the time value
  ;             given in MyPlaySound()
  ;****************************************************
  Procedure MyReleaseSound()

    ; run through each of the sounds
    ForEach(Sounds())
      ; if a particular one has expired (time-wise)
      If ElapsedMilliseconds() - Sounds()\StartTime > Sounds()\Length
        ; stop the sound, remove the sound, and delete the element 
        ; from the list
        StopSound(Sounds()\ID)
        FreeSound(Sounds()\ID)
        DeleteElement(Sounds())
      EndIf
    Next
    
 EndProcedure
  
; Keep going until the user hits ESCAPE
Repeat
   ; clear the screen
   ClearScreen(ClearColor)

   ExamineKeyboard()
   ; if the user hits the spacebar, play the sound
   If KeyboardReleased(#PB_Key_Space)
      MyPlaySound()
   EndIf

   ; put up text for information
   StartDrawing(ScreenOutput())
     DrawText(0,0,"Press the spacebar as much as you want to make explosions!")
     DrawText(0,400,"Press ESCAPE to quit")
   StopDrawing()

   FlipBuffers()
   MyReleaseSound()

Until KeyboardReleased(#PB_Key_Escape)

End

DataSection
; where we load in the explosion sound
ExplosionSound: IncludeBinary "sounds\explosion1.wav"
EndDataSection
  
; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 20
; Folding = -