;***************************************************************************
;* Migz Callo: Laser Blazer
;* Demo game for "Programming 2D Games in PureBasic"
;* 
;*     Coded By: John Logsdon with help from Derlidio Sequeira
;* Last Updated: 4/14/2005
;***************************************************************************

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Set up the constant vars
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Screen dimension constants
#Screen_Width = 800
#Screen_Height = 600

; setup our widths and heights for loading and collision checking
#TileWidth = 32         ; width of the map tiles
#TileHeight = 32        ; height of the map tiles

; determine how many columns/rows to draw per refresh
#MapColumns = 24       ; how many columns? 
#MapRows = 15          ; how many rows?

; Initialize PureBasic
If InitSprite() = 0 Or InitKeyboard() = 0 Or InitSound() = 0 Or OpenScreen(#Screen_Width,#Screen_Height,16,"Migz Callo: Laser Blazer") = 0
  MessageRequester("Error!", "Unable to Initialize Environment", #PB_MessageRequester_Ok)
  End
EndIf

Global ClearColor = RGB(0,0,0) 
Global TransparencyColor = RGB(255,0,255)
; set the appropriate mask so we have transparencies
TransparentSpriteColor(#PB_Default,TransparencyColor)


; We're using some PNG files, so we need the decoder
UsePNGImageDecoder()

; make sure we use the appropriate ogg plug-in for sounds!
UseOGGSoundDecoder()

; setup a random seed
RandomSeed(ElapsedMilliseconds())


; Declare our procedures here
Declare CheckMovementKeys()
Declare SetupCollisionPoints(TileWidth,TileHeight)
Declare MovePlayer(Direction.l,Distance.l,TileWidth,TileHeight)
Declare LoadPlayerFrames(Player_Full_Path.s,PlayerWidth.l,PlayerHeight.l,PlayerSpacer.l)
Declare LoadRobotFrames(Robot_Full_Path.s,RobotWidth.l,RobotHeight.l,RobotSpacer.l)
Declare LoadLaserFrames(Laser_Full_Path.s,LaserWidth.l,LaserHeight.l,LaserSpacer.l)
Declare AddRobot(MapX.l,MapY.l,MinX.l,MaxX.l,TileWidth,TileHeight)
Declare MoveRobots(Current_Time,MoveMap.l,Direction.l,TileWidth,TileHeight)
Declare AddLaser(Owner.l,X.l,Y.l,WX.l,WY.l,Direction.l,TileWidth,TileHeight)
Declare MoveLasers(Current_Time,MoveMap.l,Direction.l,TileWidth,TileHeight)
Declare AddHealthPak(MapX.l,MapY.l,TileWidth,TileHeight)
Declare MoveHealthPaks(Current_Time,MoveMap.l,Direction.l,TileWidth,TileHeight)
Declare AnimatePlayer(Current_Time,Animation_Time.l,Direction.l,TileWidth,TileHeight)
Declare StartLevel(Current_Level,TileWidth,TileHeight)
Declare MyPlaySound(SoundType.l)
Declare MyRemoveSound()

; User can control seeing where the collision boxes are
Global ShowCollisionPoints

; setup the viewport information
Global ViewPortWidth = #MapColumns * #TileWidth 
Global ViewPortHeight = #MapRows * #TileHeight

; set up the default x,y location of where to draw the player.  This should be
; the center of the viewport (or screen if you have no viewport)
Global ScreenX
Global ScreenY

; use this to keep an unchangeable copy of the center
Global ScreenCenterX
Global ScreenCenterY

; Store the actual pixel width/height to a set of globals
Global Map_Pixel_Width 
Global Map_Pixel_Height

; how many pixels to move the player?
; Make sure it divides evenly into TileWidth AND TileHeight
#MoveDistance = 2        

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Player variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Global PlayerWorldX 
Global PlayerWorldY
Global DrawPlayerX
Global DrawPlayerY
Global PlayerFrame
Global PlayerFacing
Global PlayerArmor
Global PlayerHealthBar
Global PlayerHit
Global PlayerHitAnimation
Global PlayerDead
Global PlayerExitX
Global PlayerExitY
Global PlayerFiring
Global PlayerFidgeting
Global PlayerSleeping

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Laser/Hit variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Global LaserSpeed
Global RobotArmorHit
Global PlayerArmorHit
Global AllowableDistance 
Global RHits

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Timers
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Global Firing_Speed = 50
Global Firing_Timer = ElapsedMilliseconds()
Global Firing_Delay = 200
Global Fidget_Timer = ElapsedMilliseconds()
Global Fidget_Interval = 5000 ; how often does the player fidget
Global Fidget_Level = 0  ; if the player stands still for too long, increase
Global Fidget_Speed = 250
Global Snore_Timer = ElapsedMilliseconds()
Global Snore_Interval = 2200 ; how often does the player fidget
Global Frame_Timer = ElapsedMilliseconds()
Global Restart_Timer = ElapsedMilliseconds()
Global Start_Timer = ElapsedMilliseconds()
Global RestartTime = 6

; Structure For the Player
Structure Player_Frame_Images
  Image.l
  X.l
  Y.l
  Width.l
  Height.l
EndStructure

; Structure For the Robot Images
Structure Robot_Frame_Images
  Image.l
  X.l
  Y.l
  Width.l
  Height.l
EndStructure

; structure for the laser images
Structure Laser_Frame_Images
  Image.l
  X.l
  Y.l
  Width.l
  Height.l
EndStructure

; Structure For the Robot Images
Structure Robots
  WorldX.l
  WorldY.l
  ScreenX.l
  ScreenY.l
  MinimumX.l
  MaximumX.l
  Direction.l
  Frame.l
  Hit.l
  Dead.l
  Armor.l
  Timer.l
  Timeout.l
  ActionTimer.l
  ActionTimeout.l
  Firing.l
  FiringTimer.l
  FiringTimeout.l
EndStructure

; setup a list for the map walls
Global NewList Robot.Robots()

; Structure For the Robot Images
Structure Lasers
  Owner.l
  WorldX.l
  WorldY.l
  ScreenX.l
  ScreenY.l
  Direction.l
  Distance.l
  Status.l
  Beam.l
  Explosion.l
  Timer.l
  Timeout.l
EndStructure

; setup a list for the map walls
Global NewList Laser.Lasers()

; Structure For the Health Paks
Structure HealthPaks
  WorldX.l
  WorldY.l
  ScreenX.l
  ScreenY.l
EndStructure

; setup a list for the map walls
Global NewList HealthPak.HealthPaks()

; setup our structure for the sounds
Structure SoundController
  ID.l 
  Length.l 
  StartTime.l
EndStructure

; make sure we have a list for the sounds
Global NewList Sounds.SoundController()

; Dimension our tile array
Global Dim Player.Player_Frame_Images(0)

; Load the tiles for the Map
LoadPlayerFrames("graphics\migzcallospritesheet.png",32,32,0)

Global HealthPak_Image = LoadSprite(#PB_Any,"graphics\healthpack.png")

; Dimension our tile array
Global Dim RobotImages.Robot_Frame_Images(0)

; Load the tiles for the Robots
LoadRobotFrames("graphics\robotspritesheet.png",32,32,0)

; Dimension our tile array
Global Dim LaserImages.Laser_Frame_Images(0)

; Load the tiles for the Lasers
LoadLaserFrames("graphics\lasers.png",8,8,0)

; load up the HUD (heads-up display) image...wraps around the map
HUD_Image = LoadSprite(#PB_Any,"graphics\migzcallohud.png")

; Load up our backdrop (the stars)
BackDrop_Image = LoadSprite(#PB_Any,"graphics\backdrop.png")

; Load up the splash screen
Splash_Image = LoadSprite(#PB_Any,"graphics\migzcallosplash.png")

; Load up the splash screen
Story_Image = LoadSprite(#PB_Any,"graphics\migzstory.png")

; Load up the splash screen
Instruction_Image = LoadSprite(#PB_Any,"graphics\migzinstructions.png")

; Load up the credits page
Credits_Image = LoadSprite(#PB_Any,"graphics\migzcallocredits.png")

; A couple of variables that let us know when we're starting a new level
Global StartingLevel
Global StartingLevelTime 

; adjust this to speed up or slow down the sprite animation
#Animation_Speed = 50     

; Load in our .OGG file
Music = LoadSound(#PB_Any,"music/spy.ogg")
If Music
   SoundVolume(Music,30)
   ; play the sound
   PlaySound(Music,1)
Else
   MessageRequester("Error!", "Unable to load music file", #PB_MessageRequester_Ok)
   End
EndIf

IncludePath "libs"

; include the keyboard and map library
XIncludeFile "maplib.pb"

; include robots lib
XIncludeFile "robots.pb"

; include lasers lib
XIncludeFile "lasers.pb"

; include health lib
XIncludeFile "health.pb"

; include player lib
XIncludeFile "player.pb"

; include levels lib
XIncludeFile "levels.pb"

; include keys lib
XIncludeFile "keys.pb"

; include sounds lib
XIncludeFile "sounds.pb"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Splash Screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Repeat
  ClearScreen(ClearColor)
  DisplaySprite(Splash_Image,0,0)
  FlipBuffers()
  ExamineKeyboard()
Until KeyboardReleased(#PB_Key_All)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Story Screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Repeat
  ClearScreen(ClearColor)
  DisplaySprite(Story_Image,0,0)
  FlipBuffers()
  ExamineKeyboard()
Until KeyboardReleased(#PB_Key_All)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Instructions Screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Repeat
  ClearScreen(ClearColor)
  DisplaySprite(Instruction_Image,0,0)
  FlipBuffers()
  ExamineKeyboard()
Until KeyboardReleased(#PB_Key_All)



; Set the current level to 1 (first level) and then start it up!
Current_Level = 1
StartLevel(Current_Level,#TileWidth,#TileHeight)
MyPlaySound(9)

; Keeping track of the number of screenshots
ScreenNum = 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MAIN GAME LOOP
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Repeat
  ; clear the screen
  ClearScreen(ClearColor)

  ; draw the backdrop image
  DisplaySprite(BackDrop_Image,2,2)

  ; Get the current time so we only have to call the ElapsedMilliseconds() once per loop
  Current_Time = ElapsedMilliseconds()
  
  ; check to see if any movement keys are being pressed (assuming we're not already firing!)
  If PlayerFiring = 0 And StartingLevel = 0
     ; see what direction we're moving, or 0 if nothing's being hit  
     Direction.l = CheckMovementKeys()
  EndIf
  
  ; check the keyboard here to see if the firing/activation key has been hit too
  ExamineKeyboard()

  ; see if the user wants a screen shot or not  
  TakeScreenie = 0
  If KeyboardPushed(#PB_Key_F10)
     ; if so set the flag for use after the flip
     TakeScreenie = 1
  EndIf
  
  ; Check for firing/activation
  If KeyboardPushed(#PB_Key_Space) And PlayerHitAnimation = 0 And PlayerDead = 0  And StartingLevel = 0
     ; if there are still Robots, then we must be shooting
     If ListSize(Robot()) > 0
        ; Make sure we're not on a ladder
        MTile = Map_CheckOnLadder(PlayerWorldX,PlayerWorldY,#TileWidth,#TileHeight)
        ; We're not
        If MTile = 0
           ; has enough time lapsed since our last firing?
           If Current_Time > Firing_Timer + Firing_Delay
              ; set the player firing flag to 1
              PlayerFiring = 1  
              ; force the direction to 0 so no running or anything can happen
              Direction = 0
              ; set the start firing frame based on which way the player is facing
              If PlayerFacing = 3
                 PlayerFrame = 31
              Else 
                 PlayerFrame = 64
              EndIf
              ; reset our firing time
              Firing_Timer = Current_Time
           EndIf 
        EndIf
    Else ; the robots must all be dead...why shoot when there's no robots?

        ; See if we're at the exit point    
        If PlayerWorldX >= PlayerExitX-5 And PlayerWorldY >= PlayerExitY-5 And PlayerWorldX <= PlayerExitX+37 And PlayerWorldY <= PlayerExitY+37 
           ; if so, we're done with this level, so go to the next one
           Current_Level = Current_Level + 1
           If Current_Level > 3
              Current_Level = 1
           EndIf
           ; play the exiting level sound
           MyPlaySound(9)
           ; start up a new level!
           StartLevel(Current_Level,#TileWidth,#TileHeight)
        EndIf
    EndIf
  EndIf

  ; Check to see what frame we should display in our next player draw
  AnimatePlayer(Current_Time,#Animation_Speed,Direction,#TileWidth,#TileHeight)

  ; move the player and see if we should scroll the map or not
  MoveMap.l = MovePlayer(Direction.l,#MoveDistance,#TileWidth,#TileHeight)
  ; if the player is standing still
  If Direction.l = 0
    ; show the map, but don't scroll it
    Map_ShowMap(0,0,0,0,#MapColumns,#MapRows,#TileWidth,#TileHeight,ShowCollisionPoints)
  Else
    ; if we want to scroll the map, do that here
    If MoveMap.l = 1
       Map_ShowMap(0,0,Direction.l,#MoveDistance,#MapColumns,#MapRows,#TileWidth,#TileHeight,ShowCollisionPoints)
    Else
       ; otherwise, just show it
       Map_ShowMap(0,0,0,0,#MapColumns,#MapRows,#TileWidth,#TileHeight,ShowCollisionPoints)
    EndIf
  EndIf

  ; Move the health paks
  MoveHealthPaks(Current_Time,MoveMap.l,Direction.l,#TileWidth,#TileHeight)

  ; Move the robots
  MoveRobots(Current_Time,MoveMap.l,Direction.l,#TileWidth,#TileHeight)  
  
  ; Move the lasers
  MoveLasers(Current_Time,MoveMap.l,Direction.l,#TileWidth,#TileHeight)

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; Drawing the player sprite
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; First let's make sure that the player's not dead and that we're not at the starting screen
  If PlayerDead > -1 And StartingLevel = 0
     ; Then draw the player
     DisplayTransparentSprite(Player(PlayerFrame)\Image,DrawPlayerX ,DrawPlayerY)
     ; next we want to draw the player's healthbar
     If StartDrawing(ScreenOutput())
        ; draw an unfilled box first to hold the bar 
        DrawingMode(4)
        Box(DrawPlayerX,DrawPlayerY-5,32,3,RGB(255,255,255))
        ; now we go back to filling the boxes we draw
        DrawingMode(0)
        ; fill it with gray
        Box(DrawPlayerX+1,DrawPlayerY-4,30,1,RGB(155,155,155))
        ; Now choose a color and distance to fill the box, based on the health of player
        If PlayerHealthBar > 15
           Box(DrawPlayerX+1,DrawPlayerY-4,PlayerHealthBar,1,RGB(0,255,0))
        EndIf
        If PlayerHealthBar > 9 And PlayerHealthBar <= 15
           Box(DrawPlayerX+1,DrawPlayerY-4,PlayerHealthBar,1,RGB(255,255,0))
        EndIf
        If PlayerHealthBar <= 9
           Box(DrawPlayerX+1,DrawPlayerY-4,PlayerHealthBar,1,RGB(255,0,0))
        EndIf
     Else
        MessageRequester("Error!", "Unable to Draw to ScreenOutput()",#PB_MessageRequester_Ok)
     EndIf
     StopDrawing()  
  EndIf
   
  ; if the player wants to see collision info, draw the bounding box here
  If ShowCollisionPoints = 1
    Map_ShowPlayerBoundingBoxes(DrawPlayerX,DrawPlayerY)
  EndIf

  ; Show our overlay image that wraps around the map viewport  
  DisplayTransparentSprite(HUD_Image,0,0)
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ; Special messages
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  If StartDrawing(ScreenOutput())
    ; if the player has died
    If PlayerDead = -1
       ; show a little message and a countdown. Then restart the level.
       If Current_Time > Restart_Timer + 1000
          RestartTime = RestartTime - 1
          If RestartTime = 0
             StartLevel(Current_Level,#TileHeight,#TileWidth)
             StartingLevel = 0
             MyPlaySound(10)
          EndIf
          Restart_Timer = Current_Time
       Else
         DrawText(ScreenCenterX-150,ScreenCenterY,"Putting the jumper cables on Migz and restarting level in " + Str(RestartTime))
       EndIf
    EndIf
    ; if the level is just starting fresh (not a restart) Then show a little new level information
    If StartingLevel = 1
       If Current_Time > Start_Timer + 1000
          StartingLevelTime = StartingLevelTime - 1
          If StartingLevelTime = 0
             StartingLevel = 0
             MyPlaySound(10)
          EndIf
          Start_Timer = Current_Time
       EndIf
       DrawText(ScreenCenterX-80,ScreenCenterY-40,"Level " + Str(Current_Level) + "...starting in " + Str(StartingLevelTime))
       DrawText(ScreenCenterX-150,ScreenCenterY,"Robots take " + Str(RHits) + " hits to destroy in this level.")
       DrawText(ScreenCenterX-65,ScreenCenterY+40,"Good luck, Migz!")
    EndIf
  Else
    MessageRequester("Error!", "Unable to Draw to ScreenOutput()",#PB_MessageRequester_Ok)
  EndIf
  StopDrawing()  

  ; nuke any sounds that have outlived their welcome  
  MyRemoveSound()  

  ; flip the screen so the user can see what happened
  FlipBuffers()

  ; if the player asked for a screenshot, handle that here
  If TakeScreenie = 1
     ; snag the full screen
     Screenie = GrabSprite(#PB_Any,0,0,800,600)
     ; update the filename so multiple screenies can be snagged per game
     Screenshot.s = "Migzscreenie" + Str(ScreenNum) + ".bmp"
     ; save out the screenshot
     SaveSprite(Screenie,Screenshot.s)
     ; reset the flag
     TakeScreenie = 0
     ; increase the image for next time
     ScreenNum = ScreenNum + 1
     ; wait a full second so we can show that something happened and so we can
     ; make sure we don't get like 20 F-10 presses!
     Delay(1000)
  EndIf
  ; handle the case where a player hits ALT-TAB and loses game focus
  ; if you don't do this, your game will crash on ALT-TAB
  If IsScreenActive() = 0
     Repeat
        FlipBuffers()
     Until IsScreenActive() = 1
  EndIf
Until KeyboardReleased(#PB_Key_Escape)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; END OF MAIN GAME LOOP
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Show credits
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Repeat
  ClearScreen(ClearColor)
  DisplaySprite(Credits_Image,0,0)
  FlipBuffers()
  ExamineKeyboard()
Until KeyboardReleased(#PB_Key_All)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; end the program
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
End

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Data - Sounds only
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
DataSection
  ; where we load in the explosion sound
IncludePath "sounds"
LaserSound: IncludeBinary "laser.wav"
HealthSound: IncludeBinary "powerup.wav"
PlayerHitSound: IncludeBinary "playerhit.wav"
PlayerDeadSound: IncludeBinary "playerdeath.wav"
RobotHitSound: IncludeBinary "robothit.wav"
RobotDeadSound: IncludeBinary "robotdeath.wav"
StartLevelSound: IncludeBinary "startlevel.wav"
ExitLevelSound: IncludeBinary "exitlevel.wav"
IntruderAlertSound: IncludeBinary "intruderalert.wav"
PlayerSnoringSound: IncludeBinary "snore.wav"
EndDataSection

      




 
; IDE Options = PureBasic 4.61 (Windows - x64)
; CursorPosition = 544
; FirstLine = 470
; Executable = C:\Krylars Kreations\Migz\code\migz_demo.exe